from typing import Optional

from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.short_role_name_repository_abc import ShortRoleNameRepositoryABC
from bot_data.model.short_role_name import ShortRoleName
from bot_data.model.short_role_name_position_enum import ShortRoleNamePositionEnum


class ShortRoleNameRepositoryService(ShortRoleNameRepositoryABC):
    def __init__(
        self,
        logger: DatabaseLogger,
        db_context: DatabaseContextABC,
        servers: ServerRepositoryABC,
    ):
        self._logger = logger
        self._context = db_context

        self._servers = servers

        ShortRoleNameRepositoryABC.__init__(self)

    @staticmethod
    def _get_value_from_result(value: any) -> Optional[any]:
        if isinstance(value, str) and "NULL" in value:
            return None

        return value

    def _short_role_name_from_result(self, sql_result: tuple) -> ShortRoleName:
        return ShortRoleName(
            self._get_value_from_result(sql_result[1]),  # name
            int(self._get_value_from_result(sql_result[2])),  # role_id
            ShortRoleNamePositionEnum(self._get_value_from_result(sql_result[3])),  # position
            self._servers.get_server_by_id(sql_result[4]),  # server
            self._get_value_from_result(sql_result[5]),  # created_at
            self._get_value_from_result(sql_result[6]),  # modified_at
            id=self._get_value_from_result(sql_result[0]),  # id
        )

    def get_short_role_names(self) -> List[ShortRoleName]:
        short_role_names = List(ShortRoleName)
        self._logger.trace(__name__, f"Send SQL command: {ShortRoleName.get_select_all_string()}")
        results = self._context.select(ShortRoleName.get_select_all_string())
        for result in results:
            self._logger.trace(__name__, f"Get short_role_name with id {result[0]}")
            short_role_names.append(self._short_role_name_from_result(result))

        return short_role_names

    def get_short_role_name_by_id(self, id: int) -> ShortRoleName:
        self._logger.trace(__name__, f"Send SQL command: {ShortRoleName.get_select_by_id_string(id)}")
        result = self._context.select(ShortRoleName.get_select_by_id_string(id))[0]

        return self._short_role_name_from_result(result)

    def find_short_role_names_by_role_id(self, role_id: int) -> List[ShortRoleName]:
        short_role_names = List(ShortRoleName)
        self._logger.trace(
            __name__,
            f"Send SQL command: {ShortRoleName.get_select_by_role_id_string(role_id)}",
        )
        results = self._context.select(ShortRoleName.get_select_by_role_id_string(role_id))
        for result in results:
            self._logger.trace(__name__, f"Get short_role_name with id {result[0]}")
            short_role_names.append(self._short_role_name_from_result(result))

        return short_role_names

    def get_short_role_names_by_server_id(self, server_id: int) -> List[ShortRoleName]:
        short_role_names = List(ShortRoleName)
        self._logger.trace(
            __name__,
            f"Send SQL command: {ShortRoleName.get_select_by_server_id_string(server_id)}",
        )
        results = self._context.select(ShortRoleName.get_select_by_server_id_string(server_id))

        for result in results:
            self._logger.trace(__name__, f"Get short_role_name with id {result[0]}")
            short_role_names.append(self._short_role_name_from_result(result))

        return short_role_names

    def add_short_role_name(self, short_role_name: ShortRoleName):
        self._logger.trace(__name__, f"Send SQL command: {short_role_name.insert_string}")
        self._context.cursor.execute(short_role_name.insert_string)

    def update_short_role_name(self, short_role_name: ShortRoleName):
        self._logger.trace(__name__, f"Send SQL command: {short_role_name.udpate_string}")
        self._context.cursor.execute(short_role_name.udpate_string)

    def delete_short_role_name(self, short_role_name: ShortRoleName):
        self._logger.trace(__name__, f"Send SQL command: {short_role_name.delete_string}")
        self._context.cursor.execute(short_role_name.delete_string)
