import { Component, OnDestroy, OnInit } from "@angular/core";
import { User } from "../../../../../../models/data/user.model";
import { LazyLoadEvent, MenuItem } from "primeng/api";
import { FormBuilder, FormControl, FormGroup } from "@angular/forms";
import { Page } from "../../../../../../models/graphql/filter/page.model";
import { Sort, SortDirection } from "../../../../../../models/graphql/filter/sort.model";
import { AuthService } from "../../../../../../services/auth/auth.service";
import { SpinnerService } from "../../../../../../services/spinner/spinner.service";
import { ToastService } from "../../../../../../services/toast/toast.service";
import { ConfirmationDialogService } from "../../../../../../services/confirmation-dialog/confirmation-dialog.service";
import { TranslateService } from "@ngx-translate/core";
import { DataService } from "../../../../../../services/data/data.service";
import { SidebarService } from "../../../../../../services/sidebar/sidebar.service";
import { ActivatedRoute } from "@angular/router";
import { AutoRoleQuery, SingleDiscordQuery } from "../../../../../../models/graphql/query.model";
import { Queries } from "../../../../../../models/graphql/queries.model";
import { catchError, debounceTime, takeUntil } from "rxjs/operators";
import { Table } from "primeng/table";
import { AutoRoleMutationResult } from "../../../../../../models/graphql/result.model";
import { Mutations } from "../../../../../../models/graphql/mutations.model";
import { Subject, throwError } from "rxjs";
import { AutoRole, AutoRoleFilter } from "../../../../../../models/data/auto_role.model";
import { ChannelType, Guild } from "../../../../../../models/data/discord.model";
import { Server } from "../../../../../../models/data/server.model";
import { ComponentWithTable } from "../../../../../../base/component-with-table";

@Component({
  selector: "app-auto-roles",
  templateUrl: "./auto-roles.component.html",
  styleUrls: ["./auto-roles.component.scss"]
})
export class AutoRolesComponent extends ComponentWithTable implements OnInit, OnDestroy {
  auto_roles: AutoRole[] = [];
  guild: Guild = { channels: [], emojis: [], roles: [] };
  channels: MenuItem[] = [];
  loading = true;

  clonedUsers: { [s: string]: User; } = {};
  isEditingNew: boolean = false;

  newAutoRoleTemplate: AutoRole = {
    createdAt: "",
    modifiedAt: ""
  };

  filterForm!: FormGroup<{
    id: FormControl<number | null>,
    channelId: FormControl<string | null>,
    channelName: FormControl<string | null>,
    messageId: FormControl<string | null>,
  }>;

  filter: AutoRoleFilter = {};
  page: Page = {
    pageSize: undefined,
    pageIndex: undefined
  };
  sort: Sort = {
    sortColumn: undefined,
    sortDirection: undefined
  };

  totalRecords: number = 0;
  private unsubscriber = new Subject<void>();
  private server: Server = {};

  query: string = Queries.autoRolesWithHistoryQuery;

  constructor(
    private authService: AuthService,
    private spinner: SpinnerService,
    private toastService: ToastService,
    private confirmDialog: ConfirmationDialogService,
    private fb: FormBuilder,
    private translate: TranslateService,
    private data: DataService,
    private sidebar: SidebarService,
    private route: ActivatedRoute
  ) {
    super("auto-role", ["id", "channel_id", "channel_name", "message_id", "rule_count"]);
  }

  public ngOnInit(): void {

    this.setFilterForm();
    this.data.getServerFromRoute(this.route).then(server => {
      this.server = server;
      this.spinner.showSpinner();
      this.data.query<SingleDiscordQuery>(Queries.guildsQuery, {
          id: server?.discordId,
          filter: {
            type: ChannelType.text
          }
        }
      ).subscribe(data => {
        if (data.discord.guilds) {
          this.guild = data.discord.guilds[0];
        }
        this.channels = this.guild.channels
          .map(x => {
            return { label: x.name, value: x };
          });
        this.spinner.hideSpinner();
      });
      this.loadNextPage();
    });
  }

  public ngOnDestroy(): void {
    this.unsubscriber.next();
    this.unsubscriber.complete();
  }


  public loadNextPage(): void {
    this.loading = true;
    this.data.query<AutoRoleQuery>(Queries.autoRolesQuery, {
        serverId: this.server.id, filter: this.filter, page: this.page, sort: this.sort
      },
      (x: { servers: Server[] }) => {
        return x.servers[0];
      }
    ).subscribe(data => {
      this.totalRecords = data.autoRoleCount;
      this.auto_roles = data.autoRoles;
      this.spinner.hideSpinner();
      this.loading = false;
    });
  }

  public setFilterForm(): void {
    this.filterForm = this.fb.group({
      id: new FormControl<number | null>(null),
      channelId: new FormControl<string | null>(null),
      channelName: new FormControl<string | null>(null),
      messageId: new FormControl<string | null>(null)
    });

    this.filterForm.valueChanges.pipe(
      takeUntil(this.unsubscriber),
      debounceTime(600)
    ).subscribe(changes => {
      if (changes.id) {
        this.filter.id = changes.id;
      } else {
        this.filter.id = undefined;
      }

      if (changes.channelId) {
        this.filter.channelId = changes.channelId;
      } else {
        this.filter.channelId = undefined;
      }

      if (changes.channelName) {
        this.filter.channelName = changes.channelName;
      } else {
        this.filter.channelName = undefined;
      }

      if (changes.messageId) {
        this.filter.messageId = changes.messageId;
      } else {
        this.filter.messageId = undefined;
      }

      if (this.page.pageSize)
        this.page.pageSize = 10;

      if (this.page.pageIndex)
        this.page.pageIndex = 0;

      this.loadNextPage();
    });
  }

  public nextPage(event: LazyLoadEvent): void {
    this.page.pageSize = event.rows ?? 0;
    if (event.first != null && event.rows != null)
      this.page.pageIndex = event.first / event.rows;
    this.sort.sortColumn = event.sortField ?? undefined;
    this.sort.sortDirection = event.sortOrder === 1 ? SortDirection.ASC : event.sortOrder === -1 ? SortDirection.DESC : SortDirection.ASC;

    this.loadNextPage();
  }

  public resetFilters(): void {
    this.filterForm.reset();
  }

  public onRowEditInit(table: Table, autoRole: AutoRole, index: number): void {
    this.clonedUsers[index] = { ...autoRole };
  }

  public onRowEditSave(table: Table, newAutoRole: AutoRole, index: number): void {
    if (this.isEditingNew && JSON.stringify(newAutoRole) === JSON.stringify(this.newAutoRoleTemplate)) {
      this.isEditingNew = false;
      this.auto_roles.splice(index, 1);
      return;
    }

    if (!newAutoRole.id && !this.isEditingNew || !newAutoRole.channelId && !newAutoRole.messageId) {
      return;
    }

    this.spinner.showSpinner();
    this.data.mutation<AutoRoleMutationResult>(Mutations.createAutoRole, {
        serverId: this.server.id,
        channelId: newAutoRole.channelId,
        messageId: newAutoRole.messageId
      }
    ).pipe(catchError(err => {
      this.isEditingNew = false;
      this.spinner.hideSpinner();
      return throwError(err);
    })).subscribe(result => {
      this.isEditingNew = false;
      this.spinner.hideSpinner();
      this.toastService.success(this.translate.instant("view.server.auto_roles.message.auto_role_create"), this.translate.instant("view.server.auto_roles.message.auto_role_create_d", { id: result.autoRole.createAutoRole?.id }));
      this.loadNextPage();
    });
  }

  public onRowEditCancel(index: number): void {
    if (this.isEditingNew) {
      this.auto_roles.splice(index, 1);
      delete this.clonedUsers[index];
      this.isEditingNew = false;
      return;
    }

    this.auto_roles[index] = this.clonedUsers[index];
    delete this.clonedUsers[index];
  }

  public deleteAutoRole(autoRole: AutoRole): void {
    this.confirmDialog.confirmDialog(
      this.translate.instant("view.server.auto_roles.message.auto_role_delete"), this.translate.instant("view.server.auto_roles.message.auto_role_delete_q", { id: autoRole.id }),
      () => {
        this.spinner.showSpinner();
        this.data.mutation<AutoRoleMutationResult>(Mutations.deleteAutoRole, {
            id: autoRole.id
          }
        ).pipe(catchError(err => {
          this.spinner.hideSpinner();
          return throwError(err);
        })).subscribe(_ => {
          this.spinner.hideSpinner();
          this.toastService.success(this.translate.instant("view.server.auto_roles.message.auto_role_deleted"), this.translate.instant("view.server.auto_roles.message.auto_role_deleted_d", { id: autoRole.id }));
          this.loadNextPage();
        });
      });


  }

  public addAutoRole(table: Table): void {
    const newAutoRole = JSON.parse(JSON.stringify(this.newAutoRoleTemplate));

    this.auto_roles = [newAutoRole, ...this.auto_roles];

    table.initRowEdit(newAutoRole);

    const index = this.auto_roles.findIndex(u => u.id == newAutoRole.id);
    this.onRowEditInit(table, newAutoRole, index);

    this.isEditingNew = true;
  }
}
